import java.awt.Canvas;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import java.awt.Toolkit;

/*            Interpolation by Newton Gregory             */
/*                www.mosismath.com                       */

public class mainFrame {

	private JFrame frmNewtonGregory;
	private JPanel pGraph; 
	
	 public int order = 6;         // number of supporting points
     public int datapoints = 1000; // number of datapoints for the interpolation
     public double deltaX = 0.1;   // sample time 

     double[] y_k = new double[order];
     double[] x_k = new double[order];
     double[] b = new double[order];
     double[] yp = new double[datapoints];
     double[] xp = new double[datapoints];
     double[] c = new double[order];
     
     private JTextField[] tbs = new JTextField[order];
     private JTextField[] tbx = new JTextField[order];
     private JTextField textBoxY1;
     private JTextField textBoxY2;
     private JTextField textBoxY3;
     private JTextField textBoxY4;
     private JTextField textBoxY5;
     private JTextField textBoxY6;
     private JTextField textBoxX1;
     private JTextField textBoxX2;
     private JTextField textBoxX3;
     private JTextField textBoxX4;
     private JTextField textBoxX5;
     private JTextField textBoxX6;
     
     public class TPoint  {
 		public float X;
         public float Y;	
 	}

     public class myCanvas extends Canvas {
     	
 		private static final long serialVersionUID = 1L;
 		
 		public void paint(Graphics g) { 		 			              
 			  drawGraph(g, datapoints, x_k[0], x_k[order - 1], true);
 		}
     }
    
     public double Binom(double n, double k)
     {
         double i, j;
         double res = 1;
         j = 0;
         if ((k != 0) && (k != n))
         {
             for (i = 0; i < k; i++)
             {
                 res = res * (n - j);
                 j++;
             }
             for (i = 2; i <= k; i++)
             {
                 res = res / i;
             }
         }
         return res;
     }

     void CalcElements(double[] y)
     {
         int i, j;
         double sign = 1;
         for (i = 0; i < y.length; i++)
         {
             c[i] = 0;
             sign = 1;
             for (j = 0; j < i + 1; j++)
             {
                 c[i] = c[i] + sign * y[y.length - j - 1] * Binom(i, j);
                 sign = -sign;
             }
             if (i % 2 != 0)
                 c[i] = -c[i];
         }
     }

     double Interpolate(double xp, int order)
     {
         int i;
         double tempYp = 0;
         for (i = 0; i < order; i++)
         {
             tempYp = tempYp + Binom(xp / deltaX, i) * c[i];
         }
         return tempYp;
     }

    private void calculate()
    {
    	int i;
       
        for (i = 0; i < order; i++)
        {
            if (tbs[i].getText() != "")
            {
                y_k[i] = Double.parseDouble(tbs[i].getText());
                x_k[i] = Double.parseDouble(tbx[i].getText());
            }
            else
            {
                y_k[i] = 0.0;
                x_k[i] = 0.0;
            }

        }
        CalcElements(y_k);
        for (i = datapoints-1; i >= 0; i--)
        {
            xp[i] = x_k[order - 1] - (double)(i) * x_k[order - 1] / (double)(datapoints);
            yp[i] = Interpolate(xp[i], order);
            xp[i] = (double)(i) * x_k[order - 1] / (double)(datapoints);
        }
    }

    
    public void drawGraph(Graphics g, int maxPoints,  double minX, double maxX,  boolean doClear)
    {
        TPoint p1, p2;
        int j;
        double maxTime = -1.0;
        double maxVal = -1.0;
        double minVal = 1.0;
        double scalefactor;
        boolean drawCeroline;
        for (j = 0; j < maxPoints - 1; j++)
        {
        	if ((xp[j] >= minX) && (xp[j] <= maxX))
            {
        		if (maxTime < xp[j])
        			maxTime = xp[j];
        		if (maxVal < yp[j])
        			maxVal = yp[j];
        		if (minVal > yp[j])
        			minVal = yp[j];       
            }
        }
        maxTime = maxTime * 1.05;
        maxTime = maxTime * 1.05;
        if (minVal < 0)
        {
            if (maxVal > 0)
            {
                if (maxVal > Math.abs(minVal))
                    scalefactor = pGraph.getHeight() / maxVal / 2.2;
                else
                    scalefactor = pGraph.getHeight() / Math.abs(minVal) / 2.2;
            }
            else
                scalefactor = 5.0;
            drawCeroline = true;
        }
        else
        {
            scalefactor = pGraph.getHeight() / (maxVal) / 1.1;
            drawCeroline = false;
        }
        p1 = new TPoint();
        p2 = new TPoint();
        if (doClear)
        	g.clearRect(0, 0, pGraph.getWidth(), pGraph.getHeight());
       
        g.setColor(new Color(0, 0, 255));
        if (drawCeroline)
        {
            p1.X = 0;
            p1.Y = (pGraph.getHeight() / 2);
            p2.X = pGraph.getWidth();
            p2.Y = (pGraph.getHeight() / 2);
            g.setColor(new Color(0, 0, 0));
            g.drawLine((int)Math.round(p1.X),(int)Math.round(p1.Y),(int)Math.round( p2.X),(int)Math.round(p2.Y)); 
        }
        g.setColor(new Color(0, 0, 255));
        if (maxTime > 0) // draw interpolated graph
        {
            for (j = 0; j < maxPoints - 1; j++)
            {     
            	if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
            		p1.X = (int)Math.round(xp[j] * pGraph.getWidth() / (double)maxTime);
            		p2.X = (int)Math.round(xp[j + 1] * pGraph.getWidth() / (double)maxTime);
            		if (drawCeroline)
            		{
            			p1.Y = (pGraph.getHeight() / 2) - (int)Math.round(yp[j] * scalefactor);
            			p2.Y = (pGraph.getHeight() / 2) - (int)Math.round(yp[j + 1] * scalefactor);
            		}
            		else
            		{
            			p1.Y = (pGraph.getHeight() - (int)Math.round(yp[j] * scalefactor));
            			p2.Y = (pGraph.getHeight() - (int)Math.round(yp[j + 1] * scalefactor));
            		}
            		g.drawLine((int)Math.round(p1.X),(int)Math.round(p1.Y),(int)Math.round( p2.X),(int)Math.round(p2.Y)); 
                }	
            }
            g.setColor(new Color(255, 0, 0));
            for (j = 0; j < order; j++) // mark reference points
            {
         	   p1.X = (int)Math.round(x_k[j] * pGraph.getWidth() / (double)maxTime);
                if (drawCeroline)
                    p1.Y = (pGraph.getHeight() / 2) - (int)Math.round(y_k[j] * scalefactor);
                else
                    p1.Y = (pGraph.getHeight() - (int)Math.round(y_k[j] * scalefactor));
                g.drawOval((int)Math.round(p1.X-2), (int)Math.round(p1.Y-2), 4, 4);
            }
        }
    }

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					mainFrame window = new mainFrame();
					window.frmNewtonGregory.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public mainFrame() {
		int i;
		initialize();
		tbs[0] = textBoxY1;
		tbs[1] = textBoxY2;
		tbs[2] = textBoxY3;
		tbs[3] = textBoxY4;
		tbs[4] = textBoxY5;
		tbs[5] = textBoxY6;
		tbx[0] = textBoxX1;
		tbx[1] = textBoxX2;
		tbx[2] = textBoxX3;
		tbx[3] = textBoxX4;
		tbx[4] = textBoxX5;
		tbx[5] = textBoxX6;
		
		// some sample values for y
        y_k[0] = 0.5;
        y_k[1] = 11.7;
        y_k[2] = 14.8;
        y_k[3] = 4.0;
        y_k[4] = 2.2;
        y_k[5] = 0.2;
       // timestamp to y value
       for (i = 1; i <= order; i++)
       {
           x_k[i - 1] = deltaX * (double)i;
       }
       
       for (i = 0; i < order; i++)
       {
    	
           tbs[i].setText(Double.toString(y_k[i]));
           tbx[i].setText(Double.toString(x_k[i]));

       }
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		frmNewtonGregory = new JFrame();
		frmNewtonGregory.setTitle("Newton Gregory");
		frmNewtonGregory.setIconImage(Toolkit.getDefaultToolkit().getImage(mainFrame.class.getResource("/Ressource/Icon.png")));
		frmNewtonGregory.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frmNewtonGregory.setBounds(100, 100, 840, 536);
		frmNewtonGregory.getContentPane().setLayout(null);
		pGraph = new JPanel();
		final Canvas canvas = new myCanvas();
		canvas.setBackground(Color.WHITE);
		pGraph.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent arg0) {
				canvas.setBounds(0, 0, pGraph.getWidth(), pGraph.getHeight());
			}
		});
		pGraph.setBackground(Color.WHITE);
		pGraph.setBounds(10, 11, 800, 393);
		frmNewtonGregory.getContentPane().add(pGraph);
		pGraph.add(canvas);
		pGraph.setLayout(null);
		
		textBoxY1 = new JTextField();
		textBoxY1.setColumns(10);
		textBoxY1.setBounds(49, 416, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxY1);
		
		textBoxY2 = new JTextField();
		textBoxY2.setColumns(10);
		textBoxY2.setBounds(121, 416, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxY2);
		
		textBoxY3 = new JTextField();
		textBoxY3.setColumns(10);
		textBoxY3.setBounds(193, 416, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxY3);
		
		textBoxY4 = new JTextField();
		textBoxY4.setColumns(10);
		textBoxY4.setBounds(265, 416, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxY4);
		
		textBoxY5 = new JTextField();
		textBoxY5.setColumns(10);
		textBoxY5.setBounds(337, 416, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxY5);
		
		textBoxY6 = new JTextField();
		textBoxY6.setColumns(10);
		textBoxY6.setBounds(409, 416, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxY6);
		
		textBoxX1 = new JTextField();
		textBoxX1.setColumns(10);
		textBoxX1.setBounds(49, 457, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxX1);
		
		textBoxX2 = new JTextField();
		textBoxX2.setColumns(10);
		textBoxX2.setBounds(121, 457, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxX2);
		
		textBoxX3 = new JTextField();
		textBoxX3.setColumns(10);
		textBoxX3.setBounds(193, 457, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxX3);
		
		textBoxX4 = new JTextField();
		textBoxX4.setColumns(10);
		textBoxX4.setBounds(265, 457, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxX4);
		
		textBoxX5 = new JTextField();
		textBoxX5.setColumns(10);
		textBoxX5.setBounds(337, 457, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxX5);
		
		textBoxX6 = new JTextField();
		textBoxX6.setColumns(10);
		textBoxX6.setBounds(409, 457, 62, 30);
		frmNewtonGregory.getContentPane().add(textBoxX6);
		
		JLabel lblNewLabel = new JLabel("Y");
		lblNewLabel.setFont(new Font("Tahoma", Font.PLAIN, 14));
		lblNewLabel.setBounds(20, 423, 46, 14);
		frmNewtonGregory.getContentPane().add(lblNewLabel);
		
		JLabel lblX = new JLabel("X");
		lblX.setFont(new Font("Tahoma", Font.PLAIN, 14));
		lblX.setBounds(20, 464, 46, 14);
		frmNewtonGregory.getContentPane().add(lblX);
		
		JButton btCalc = new JButton("Calculate");
		btCalc.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				calculate();
				canvas.repaint();
			}
		});
		btCalc.setBounds(698, 415, 112, 72);
		frmNewtonGregory.getContentPane().add(btCalc);
	}
}
